import { AppConfiguration } from './AppConfiguration';
import { IContainer, LoadType } from '@ts-ioc/core';
import { IApplicationBuilder, CustomRegister } from './IApplicationBuilder';
import { ModuleBuilder } from './ModuleBuilder';
/**
 * application builder.
 *
 * @export
 * @class Default ApplicationBuilder
 * @extends {ModuleBuilder}
 * @template T
 */
export declare class DefaultApplicationBuilder extends ModuleBuilder implements IApplicationBuilder {
    baseURL: string;
    protected globalConfig: Promise<AppConfiguration>;
    protected globalModules: LoadType[];
    protected customRegs: CustomRegister[];
    root: IContainer;
    constructor(baseURL?: string);
    /**
     * use configuration.
     *
     * @param {(string | AppConfiguration)} [config]
     * @returns {this} global config for this application.
     * @memberof Bootstrap
     */
    useConfiguration(config?: string | AppConfiguration, container?: IContainer): this;
    /**
     * use module as global Depdences module.
     *
     * @param {...LoadType[]} modules
     * @returns {this}
     * @memberof PlatformServer
     */
    use(...modules: LoadType[]): this;
    registerConfgureDepds(container: IContainer, config: AppConfiguration): Promise<AppConfiguration>;
    protected mergeGlobalConfig(globalCfg: AppConfiguration, moduleCfg: AppConfiguration): AppConfiguration;
    /**
     * register ioc exts
     *
     * @protected
     * @param {IContainer} container
     * @param {AppConfiguration} config
     * @returns {Promise<IContainer>}
     * @memberof ApplicationBuilder
     */
    protected registerExts(container: IContainer, config: AppConfiguration): Promise<IContainer>;
    protected bindAppConfig(config: AppConfiguration): AppConfiguration;
    protected getDefaultConfig(): AppConfiguration;
    static classAnnations: any;
}
