"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var IBootstrapBuilder_1 = require("./IBootstrapBuilder");
var core_1 = require("@ts-ioc/core");
/**
 * bootstrap base builder.
 *
 * @export
 * @class BootstrapBuilder
 * @implements {IBootstrapBuilder}
 * @template T
 */
var BootstrapBuilder = /** @class */ (function () {
    function BootstrapBuilder() {
    }
    BootstrapBuilder_1 = BootstrapBuilder;
    BootstrapBuilder.prototype.build = function (iocModule, data) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var builder, container, instance;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        builder = this.getBuilder(iocModule);
                        if (!iocModule.bootstrap) {
                            iocModule.bootstrap = builder.getBootstrapToken(iocModule);
                        }
                        if (!iocModule.bootstrap) {
                            throw new Error('cant not find bootstrap token.');
                        }
                        container = iocModule.container;
                        if (!container) {
                            throw new Error('cant not find container.');
                        }
                        instance = container.resolve(iocModule.bootstrap, data);
                        return [4 /*yield*/, builder.buildStrategy(instance, iocModule)];
                    case 1:
                        instance = _a.sent();
                        iocModule.bootInstance = instance;
                        return [2 /*return*/, instance];
                }
            });
        });
    };
    BootstrapBuilder.prototype.getBootstrapToken = function (iocModule) {
        return iocModule.bootstrap || iocModule.moduleToken;
    };
    /**
     * bundle instance via config.
     *
     * @param {T} instance
     * @param {IocModule<any>} config
     * @returns {Promise<any>}
     * @memberof IModuleBuilder
     */
    BootstrapBuilder.prototype.buildStrategy = function (instance, iocModule) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/, instance];
            });
        });
    };
    BootstrapBuilder.prototype.getBuilder = function (iocModule) {
        if (!iocModule.bootBuilder) {
            var builder = void 0;
            if (iocModule.moduleConfig.bootBuilder) {
                builder = this.getBuilderViaConfig(iocModule.moduleConfig.bootBuilder, iocModule.container);
            }
            if (!builder) {
                builder = this;
            }
            iocModule.bootBuilder = builder;
        }
        return iocModule.bootBuilder;
    };
    BootstrapBuilder.prototype.getBuilderViaConfig = function (builder, container) {
        if (core_1.isToken(builder)) {
            return container.resolve(builder);
        }
        else if (builder instanceof BootstrapBuilder_1) {
            return builder;
        }
        return null;
    };
    // protected getBuilderViaToken(mdl: Token<T>, container: IContainer): IBootstrapBuilder<T> {
    //     if (isToken(mdl)) {
    //         let taskType = isClass(mdl) ? mdl : container.getTokenImpl(mdl);
    //         if (taskType) {
    //             let meta = lang.first(getTypeMetadata<ModuleConfiguration<T>>(this.getDecorator(), taskType));
    //             if (meta && meta.builder) {
    //                 return isToken(meta.builder) ? container.resolve(meta.builder) : meta.builder;
    //             }
    //         }
    //     }
    //     return null;
    // }
    BootstrapBuilder.classAnnations = { "name": "BootstrapBuilder", "params": { "constructor": [], "build": ["iocModule", "data"], "getBootstrapToken": ["iocModule"], "buildStrategy": ["instance", "iocModule"], "getBuilder": ["iocModule"], "getBuilderViaConfig": ["builder", "container"] } };
    BootstrapBuilder = BootstrapBuilder_1 = tslib_1.__decorate([
        core_1.Singleton(IBootstrapBuilder_1.BootstrapBuilderToken),
        tslib_1.__metadata("design:paramtypes", [])
    ], BootstrapBuilder);
    return BootstrapBuilder;
    var BootstrapBuilder_1;
}());
exports.BootstrapBuilder = BootstrapBuilder;

//# sourceMappingURL=sourcemaps/BootstrapBuilder.js.map
