import { Token, LoadType, InjectToken, IContainer } from '@ts-ioc/core';
import { AppConfiguration } from './AppConfiguration';
import { IModuleBuilder } from './IModuleBuilder';
/**
 * custom define module.
 */
export declare type CustomRegister = (container: IContainer, config?: AppConfiguration, builder?: IApplicationBuilder) => Token<any>[] | Promise<Token<any>[]>;
export declare const ApplicationBuilderToken: InjectToken<IApplicationBuilder>;
/**
 * application builder.
 *
 * @export
 * @interface IApplicationBuilder
 * @extends {IModuleBuilder}
 * @template T
 */
export interface IApplicationBuilder extends IModuleBuilder {
    /**
     * use custom configuration.
     *
     * @param {(string | AppConfiguration)} [config]
     * @param {IContainer} [container]
     * @returns {this}
     * @memberof IApplicationBuilder
     */
    useConfiguration(config?: string | AppConfiguration, container?: IContainer): this;
    /**
     * use module
     *
     * @param {...LoadType[]} modules
     * @returns {this}
     * @memberof IApplicationBuilder
     */
    use(...modules: LoadType[]): this;
}
