import { IContainer, Token, Type, Providers, IContainerBuilder } from '@ts-ioc/core';
import { IModuleBuilder } from './IModuleBuilder';
import { ModuleConfigure } from './ModuleConfiguration';
import { ModuleType, IocModule, MdlInstance, DIModuleType } from './ModuleType';
import { IBootstrapBuilder } from './IBootstrapBuilder';
/**
 * module builder
 *
 * @export
 * @class ModuleBuilder
 * @implements {IModuleBuilder}
 * @template T
 */
export declare class ModuleBuilder implements IModuleBuilder {
    constructor();
    /**
     * get container of the module.
     *
     * @param {(ModuleType | ModuleConfigure)} token module type or module configuration.
     * @param {IContainer} [defaultContainer] set default container or not. not set will create new container.
     * @returns {IContainer}
     * @memberof ModuleBuilder
     */
    getContainer(token: ModuleType | ModuleConfigure, defaultContainer?: IContainer): IContainer;
    createContainer(): IContainer;
    protected containerBuilder: IContainerBuilder;
    getContainerBuilder(): IContainerBuilder;
    protected createContainerBuilder(): IContainerBuilder;
    /**
     * build module.
     *
     * @param {(DIModuleType<TM> | ModuleConfigure)} [token]
     * @returns {Promise<MdlInstance<TM>>}
     * @memberof ModuleBuilder
     */
    build<TM>(token: DIModuleType<TM> | ModuleConfigure, defaultContainer?: IContainer): Promise<MdlInstance<TM>>;
    protected createModuleInstance(token: Token<any>, container: IContainer): MdlInstance<any>;
    protected getBootstrapBuilder(container: IContainer, cfg: ModuleConfigure): IBootstrapBuilder;
    /**
     * bootstrap module.
     *
     * @param {(DIModuleType<TM> | ModuleConfigure)} token
     * @param {IContainer} [defaultContainer]
     * @memberof ModuleBuilder
     */
    bootstrap<TM>(token: DIModuleType<TM> | ModuleConfigure, defaultContainer?: IContainer): Promise<MdlInstance<TM>>;
    protected createBootstrap(iocModule: IocModule<any>): Promise<any>;
    importModule(token: ModuleType | ModuleConfigure, container: IContainer): Promise<IContainer>;
    getDecorator(): string;
    /**
     * get configuration.
     *
     * @returns {ModuleConfigure}
     * @memberof ModuleBuilder
     */
    getConfigure(token?: ModuleType | ModuleConfigure, container?: IContainer): ModuleConfigure;
    registerDepdences(container: IContainer, config: ModuleConfigure): Promise<ModuleConfigure>;
    protected getBootstrapToken(cfg: ModuleConfigure, token?: Token<any>): Token<any>;
    protected importConfigExports(container: IContainer, providerContainer: IContainer, cfg: ModuleConfigure): Promise<IContainer>;
    protected registerConfgureDepds(container: IContainer, config: ModuleConfigure): Promise<ModuleConfigure>;
    protected getMetaConfig(bootModule: Type<any>): ModuleConfigure;
    protected isIocExt(token: Type<any>): boolean;
    protected isDIModule(token: Type<any>): boolean;
    protected registerExts(container: IContainer, config: ModuleConfigure): Promise<IContainer>;
    protected bindProvider(container: IContainer, providers: Providers[]): Token<any>[];
    static classAnnations: any;
}
