import { ClassMetadata, Token, MetadataAdapter, MetadataExtends, ITypeDecorator } from '@ts-ioc/core';
import { IModuleBuilder } from '../IModuleBuilder';
import { ModuleConfiguration } from '../ModuleConfiguration';
import { IBootstrapBuilder } from '../IBootstrapBuilder';
/**
 * DI module metadata.
 *
 * @export
 * @interface DIModuleMetadata
 * @extends {ModuleConfiguration<any>}
 * @extends {ClassMetadata}
 */
export interface DIModuleMetadata extends ModuleConfiguration<any>, ClassMetadata {
    /**
     * custom decorator type.
     *
     * @type {string}
     * @memberof DIModuleMetadata
     */
    decorType?: string;
}
/**
 * DIModule decorator, use to define class as DI Module.
 *
 * @export
 * @interface IDIModuleDecorator
 * @extends {ITypeDecorator<T>}
 * @template T
 */
export interface IDIModuleDecorator<T extends DIModuleMetadata> extends ITypeDecorator<T> {
    /**
     * DIModule decorator, use to define class as DI Module.
     *
     * @DIModule
     *
     * @param {T} [metadata] bootstrap metadate config.
     */
    (metadata: T): ClassDecorator;
}
/**
 * create bootstrap decorator.
 *
 * @export
 * @template T
 * @param {string} decorType
 * @param {(Token<IModuleBuilder> | IModuleBuilder)} [builder]
 * @param {(Token<IBootstrapBuilder<T>> | IBootstrapBuilder<T>)} [bootBuilder]
 * @param {InjectToken<IApplication>} provideType default provide type.
 * @param {MetadataAdapter} [adapter]
 * @param {MetadataExtends<T>} [metadataExtends]
 * @returns {IDIModuleDecorator<T>}
 */
export declare function createDIModuleDecorator<T extends DIModuleMetadata>(decorType: string, builder?: Token<IModuleBuilder> | IModuleBuilder, bootBuilder?: Token<IBootstrapBuilder> | IBootstrapBuilder, provideType?: Token<any>, adapter?: MetadataAdapter, metadataExtends?: MetadataExtends<T>): IDIModuleDecorator<T>;
/**
 * DIModule Decorator, definde class as DI module.
 *
 * @DIModule
 */
export declare const DIModule: IDIModuleDecorator<DIModuleMetadata>;
